package com.xxl.job.admin.controller;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.xxl.job.admin.controller.interceptor.PermissionInterceptor;
import com.xxl.job.admin.core.exception.XxlJobException;
import com.xxl.job.admin.core.model.XxlJobGroup;
import com.xxl.job.admin.core.model.XxlJobInfo;
import com.xxl.job.admin.core.model.XxlJobUser;
import com.xxl.job.admin.core.route.ExecutorRouteStrategyEnum;
import com.xxl.job.admin.core.scheduler.MisfireStrategyEnum;
import com.xxl.job.admin.core.scheduler.ScheduleTypeEnum;
import com.xxl.job.admin.core.thread.JobScheduleHelper;
import com.xxl.job.admin.core.util.I18nUtil;
import com.xxl.job.admin.dao.XxlJobGroupDao;
import com.xxl.job.admin.service.XxlJobService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.enums.ExecutorBlockStrategyEnum;
import com.xxl.job.core.glue.GlueTypeEnum;
import com.xxl.job.core.util.DateUtil;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
@RequestMapping({"/jobinfo"})
public class JobInfoController {
  private static Logger logger = LoggerFactory.getLogger(com.xxl.job.admin.controller.JobInfoController.class);
  
  @Resource
  private XxlJobGroupDao xxlJobGroupDao;
  
  @Resource
  private XxlJobService xxlJobService;
  
  @RequestMapping
  public String index(HttpServletRequest request, Model model, @RequestParam(required = false, defaultValue = "-1") int jobGroup) {
    model.addAttribute("ExecutorRouteStrategyEnum", ExecutorRouteStrategyEnum.values());
    model.addAttribute("GlueTypeEnum", GlueTypeEnum.values());
    model.addAttribute("ExecutorBlockStrategyEnum", ExecutorBlockStrategyEnum.values());
    model.addAttribute("ScheduleTypeEnum", ScheduleTypeEnum.values());
    model.addAttribute("MisfireStrategyEnum", MisfireStrategyEnum.values());
    List<XxlJobGroup> jobGroupList_all = this.xxlJobGroupDao.findAll();
    List<XxlJobGroup> jobGroupList = PermissionInterceptor.filterJobGroupByRole(request, jobGroupList_all);
    if (jobGroupList == null || jobGroupList.size() == 0)
      throw new XxlJobException(I18nUtil.getString("jobgroup_empty")); 
    model.addAttribute("JobGroupList", jobGroupList);
    model.addAttribute("jobGroup", Integer.valueOf(jobGroup));
    return "jobinfo/jobinfo.index";
  }
  
  @RequestMapping({"/pageList"})
  @ResponseBody
  public Map<String, Object> pageList(@RequestParam(required = false, defaultValue = "0") int start, @RequestParam(required = false, defaultValue = "10") int length, int jobGroup, int triggerStatus, String jobDesc, String executorHandler, String author) {
    return this.xxlJobService.pageList(start, length, jobGroup, triggerStatus, jobDesc, executorHandler, author);
  }
  
  @RequestMapping({"/add"})
  @ResponseBody
  public ReturnT<String> add(HttpServletRequest request, XxlJobInfo jobInfo) {
    PermissionInterceptor.validJobGroupPermission(request, jobInfo.getJobGroup());
    XxlJobUser loginUser = PermissionInterceptor.getLoginUser(request);
    return this.xxlJobService.add(jobInfo, loginUser);
  }
  
  @RequestMapping({"/update"})
  @ResponseBody
  public ReturnT<String> update(HttpServletRequest request, XxlJobInfo jobInfo) {
    PermissionInterceptor.validJobGroupPermission(request, jobInfo.getJobGroup());
    XxlJobUser loginUser = PermissionInterceptor.getLoginUser(request);
    return this.xxlJobService.update(jobInfo, loginUser);
  }
  
  @RequestMapping({"/remove"})
  @ResponseBody
  public ReturnT<String> remove(int id) {
    return this.xxlJobService.remove(id);
  }
  
  @RequestMapping({"/stop"})
  @ResponseBody
  public ReturnT<String> pause(int id) {
    return this.xxlJobService.stop(id);
  }
  
  @RequestMapping({"/start"})
  @ResponseBody
  public ReturnT<String> start(int id) {
    return this.xxlJobService.start(id);
  }
  
  @RequestMapping({"/trigger"})
  @ResponseBody
  public ReturnT<String> triggerJob(HttpServletRequest request, int id, String executorParam, String addressList) {
    XxlJobUser loginUser = PermissionInterceptor.getLoginUser(request);
    return this.xxlJobService.trigger(loginUser, id, executorParam, addressList);
  }
  
  @RequestMapping({"/nextTriggerTime"})
  @ResponseBody
  public ReturnT<List<String>> nextTriggerTime(String scheduleType, String scheduleConf) {
    XxlJobInfo paramXxlJobInfo = new XxlJobInfo();
    paramXxlJobInfo.setScheduleType(scheduleType);
    paramXxlJobInfo.setScheduleConf(scheduleConf);
    List<String> result = new ArrayList<>();
    try {
      Date lastTime = new Date();
      for (int i = 0; i < 5; ) {
        lastTime = JobScheduleHelper.generateNextValidTime(paramXxlJobInfo, lastTime);
        if (lastTime != null) {
          result.add(DateUtil.formatDateTime(lastTime));
          i++;
        } 
      } 
    } catch (Exception e) {
      logger.error("nextTriggerTime error. scheduleType = {}, scheduleConf= {}", new Object[] { scheduleType, scheduleConf, e });
      return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid") + e.getMessage());
    } 
    return new ReturnT(result);
  }
  
  @RequestMapping({"/getById"})
  @ResponseBody
  public XxlJobInfo getById(int id) {
    return this.xxlJobService.getById(id);
  }
}
